;************************************************************************************************
;* DCC Lokdekoder mit ATtiny15																	*
;*																								*
;*																								*
;*	RESET  		      PB5 = 1	8 = VCC 														*
;*	Licht Rckwrts   PB4 = 2	7 = PB2 DCC Eingang												*
;*	Licht Vorwrts    PB3 = 3	6 = PB1 Ausgang Motor PWM										*
;*       			  GND = 4	5 = PB0 Ausgang Vorwrts/Rckwrts 								*
;*																								*
;************************************************************************************************

.device attiny15
.INCLUDE "tn15def.inc"

;********************************
;* FUSE info:					*
;*								*
;* BODLEVEL:	ja				*
;* BODEN:		ja				*
;* SPIEN:		ja				*
;* RSTDISBL:	nein			*
;* CKSEL1:		nein			*
;* CKSEL0:		nein			*
;******************************** 

;********************************
;* Konstantendef.				*
;********************************

.equ 	EEOSCCAL 	= 	63	; Speicherplatz OSCCAL im EEPROM
.equ 	EEAdresse 	=	1	; Speicherplatz eigene Adresse im EEPROM
.equ 	EEStart 	= 	2	; Speicherplatz Start-Speed im EEPROM
.equ 	EEIncWert 	= 	3	; Speicherplatz Beschleunigungskonstante im EEPROM
.equ 	EEDecWert 	= 	4	; Speicherplatz Verzgerungskonstante im EEPROM
.equ 	EEV_max 	= 	5	; Speicherplatz Hchstgeschwindigkeit im EEPROM
.equ	EEFL_PWM	=	6	; Speicherplatz Licht PWM im EEPROM
.equ	EEad_V_ist	=	7	; Speicherplatz EEPROM Adresse mit V_ist im EEPROM bit0 mit Richtung berschrieben!
.equ	EEPWM_Peri	=	9	; Speicherplatz Motor PWM Periode
.equ	EEcv29		= 	29	; Speicherplatz Konfigurationsregister

;Flagregister
.equ	resetbit	=	0	; ist nach dec reset gesetzt
.equ	progsw1		=	1	; progschalter 1 zum prog. mit roco lokmaus
.equ	progsw2		=	2	; progschalter 2
.equ	Vmerken		=	3
.equ	bit			=	4
.equ	neuesbit	=	5
.equ	messen		=	6
.equ	keinDCC		=	7	; fr analogbetrieb(noch nicht aktiv)

;DCCregister
.equ	F1			=	0
.equ	F2			=	1
.equ	F3			=	2
.equ	F4			=	3
.equ	FL			=	4
.equ	Richtung	=	5
.equ	Richtung_inv=	6
.equ	Fahrstufen	=	7

;********************************
;* Registerdef.					*
;********************************

.def 	Adresse 	= 	r0	; eigene Adresse
.def 	V_IST 		= 	r1	; momentane Geschwindigkeit
.def 	V_SOLL	 	= 	r2	; Soll Geschwindigkeit
.def	IncWert		=	r3	; Beschleunigungsvariable
.def	DecWert		=	r4	; Bremsvariable
.def 	IncCnt 		= 	r5	; Beschleunigungszhler/Bremszhler

.def	FL_PWM		=	r7	; Licht PWM in % 0=100
.def	V_start		=	r8	; Startspannung
.def 	PROGTEST 	= 	R9	; Testbyte fr CV Schreiben
.def	byteA		=	r10	; Empfangsbytespeicher
.def	byteB		=	r11
.def	byteC		=	r12
.def	byteD		=	r13
.def	byteE		=	r14
.def	byteF		=	r15
.def	temp		=	r16	; temp register
.def	null		=	r17	; register mit wert null
.def	PWM_aus		=	r18	; TCCR1 PWM Ausgang getrennt
.def	PWM_an		=	r19	; TCCR1 PWM ein
.def	fktsreg		=	r20	; sreg sicherungsregister in FKTset Routine
.def	inttemp		=	r21	; interrupt tempregister
.def	intsreg		=	r22	; sreg sicherungsregister
.def	flag		=	r23	; Flagregister
							;	Bit0 = 1 -> resetbit
							;	Bit1 = 1 -> CV29bit6=1 zur Progumschaltung Licht PWM fr RocoMaus
							;	Bit2 = 1 -> CV29bit5=1 zur Progumschaltung Motor PWM fr RocoMaus
							;	Bit3 = 1 -> Vsoll im EEProm merken
							;	Bit4     -> das neue bit
							;	Bit5 = 1 -> neues bit da	
							;	Bit6 = 1 -> bitzeit messen aktiv
							;	Bit7 = 1 -> lnger als 10000us keine bits eingetroffen
.def	bitzaehler	=	r24	; Bitzaehler beim Empfang
.def	eead		=	r25	; eepromaddresse
.def 	DCCReg 		= 	r26	; DCC Register fr Funktionen und Fahrtrichtung
							;	Bit7 = 1 -> 28 Fahrstufen (0 = 14 Fahrstufen)
							;	Bit6 = 1 -> Fahrtrichtung reversiert
							;	Bit5 Fahrtrichtung 1=vorwrts, 0=rckwrts
							;   Bit4 FL Licht ein/aus	
							;	Bit3 F4 	
							;	Bit2 F3
							;	Bit1 F2
							;	Bit0 F1
.def	V_max		=	r27	; Begrenzung v_max auf 50/62,5%/75/88,5%/100%
.def	PWMzaehler	=	r28	; Zaehler fr Licht PWM
.def	EEV_ist		=	r29	; EE Adresse in der Vist im EEPROM gespeichert wird
.def 	GrundCnt	= 	r30	; Zhler fr Grundverzgerung

;************************************************************************************************
;*  ab hier Programmcode																		*
;************************************************************************************************

.CSEG
.ORG $0000

;************************************
;* Reset- und Interrupt-Vektoren	*
;************************************

		rjmp 	RESET 		; Reset Einsprung
		rjmp 	EXT_INT0 	; Sprung zum Eingangsinterrupt
		reti 	;PIN_CHANGE ; Pin change Einsprung
		reti 	;TIM1_CMP 	; Timer1 compare match Einsprung
		reti 	;TIM1_OVF 	; Timer1 overflow Einsprung
		rjmp 	TIM0_OVF 	; Timer0 overflow Einsprung
		reti 	;EE_RDY 	; EEPROM Ready Einsprung
		reti 	;ANA_COMP 	; Analog Comparator Einsprung
		reti	;ADC 		; ADC Conversion Einsprung

;************************************************************************************************
;*						Interruptserviceroutinen												*
;************************************************************************************************
;************************************
;*	Eingangsinterrupt				*
;************************************
; wird bei jeder fallenden Flanke an int0/b2 ausgelst
; timer0 auf 78us vor berlauf setzen ( Highbit nind. 52us+50% )
; dazu teiler auf 1 (0,625us) und zhler auf 255-(78/0,625 ca 125)
; zur sicherheit berlaufintbit von timer0 noch lschen
; knnte ja inzwischen gesetzt sein

EXT_INT0:	
		in		intsreg,sreg		;sreg sichern
		clr		inttemp				;timer anhalten
		out		tccr0,inttemp
		ldi		inttemp,146			;Zhler setzen ((255-125)+16 die beim timerneustart schon verganngen sind)
		out		tcnt0,inttemp
		in		inttemp,tifr		;berlauf timer0 lschen
		cbr		inttemp,0b00000010
		out		tifr,inttemp
		sbr		flag,(1<<messen)	;fr bit messen ein
		cbr		flag,(1<<keindcc)	;bit fr keine befehle da lschen
		ldi		inttemp,0b00000001	;timer mit Teiler auf ck starten
		out		tccr0,inttemp
		out		sreg,intsreg		;sreg zrck
		reti						;und fertig

;************************************
;*		Timer_0_interrupt			*
;************************************
; wird ca 78us nach ext_int ausgelst und liest dann bit von int0/b2 ein
; danach max 10000us auf neuen ext_int warten
; dazu teiler auf 64 und zhler auf 5

TIM0_OVF:		
		in		intsreg,sreg		;sreg sichern
		clr		inttemp				;timer anhalten
		out		tccr0,inttemp
		in		inttemp,pinb		;port einlesen
		sbrs	flag,messen			;wenn messen aktiv 1x springen
		rjmp	warten_aktiv		;sonst gehe zu
		com		inttemp				;port umdrehen
		bst		inttemp,2			;bit b2 holen
		bld		flag,bit			;und in flag bit0 sichern
		sbr		flag,(1<<neuesbit)	;neues bit da setzen
		cbr		flag,(1<<messen)	;flag fr messen aus
		ldi		inttemp,5			;Zhler auf 5
		out		tcnt0,inttemp
		ldi		inttemp,0b00000011	;timer mit Teiler auf ck/64 starten
		out		tccr0,inttemp
		out		sreg,intsreg
		reti
warten_aktiv:						;ca 10000us kein bit mehr eingetroffen
		sbr		flag,(1<<keinDCC)	;bit fr keine befehle setzen
		out		sreg,intsreg
		reti

;************************************************************************************************
;*		Unterprogramme																			*
;************************************************************************************************
;************************************
;*		EEprom byte lesen			*
;************************************
;liest ein Byte aus Addresse eead in temp ein zum CVregister lesen

eepromlesen:	
		sbic 	EECR,EEWE			; wird noch geschrieben ? EECR Bit1(EEWE)=1
        rjmp 	PC-1  		    	; dann Schleife
		out		EEAR,eead			; eeprom addresse low setzen bei kleinen eeprom eear bei groem eearl	
		sbi		EECR,EERE 			; Setze EEProm-Lesebefehl EECR Bit0(EERE)=1
		;sbic	EECR,EERE			; warten auf EEprom Daten
		;rjmp	PC-1
		in		temp,eedr			; daten einlesen
		ret

;************************************
;*		EEprom byte schreiben		*
;************************************
;schreibt ein Byte aus temp in Addresse eead zum CVregister schreiben

eepromschreiben:
		sbic 	EECR,EEWE			; wird noch geschrieben ? EECR Bit1(EEWE)=1
        rjmp 	PC-1  		    	; dann Schleife
		out		EEAR,eead			; bei kleinen eeprom eear bei groem eearl
		out		EEDR,temp			; und daten setzen
        cli       			     	; alle Interrupts ausschalten
        sbi 	EECR,EEMWE 			; Setze EEProm Master-Schreibbit 
        sbi 	EECR,EEWE		  	; Setze EEProm Schreib-Befehl
;        sbic 	EECR,EEWE 			; wird momentan noch geschrieben ?
;        rjmp 	PC-1		      	; Schleife 
        sei  			          	; alle Interrupts wieder einschalten
        ret							; Rcksprung

;************************************
;*		Funktionen setzen			*
;************************************
;wird zyklisch aus dem Hauptprogramm aufgerufen und setzt:
;1.MotorPWM ber Timer1 und MotorRichtung an PB2
;2.erzeugt Licht PWM genauigkeit ist nicht sehr hoch, hngt vom Hauptprogramm ab, reicht fr licht aus
; und setzt die Ausgnge PB3 und PB4
;3.setzt die Funktionsausgnge wenn Portpins vorhanden

fktset:	in		fktsreg,sreg		; sreg sichern
		inc		GrundCNT			; MotorPWM set wird nur bei jedem 96. Aufruf ausgefhrt
		tst		GrundCNT
		brne	lichtset			; sonst gleich zu..
		ldi		GrundCNT,160		; GrundCNT neu setzen auf 256-96
		cp 		V_IST,V_soll		; vergleiche IST <> SOLL
	  	breq 	lichtset			; wenn Wert gleich weiter mit lichtset
	  	brsh 	DEC_V				; wenn Wert grer gleich
	  	brlo 	INC_V				; wenn Wert kleiner			
	  	  	  	  
INC_V: 	inc		IncCNT
		cp 		IncCNT,IncWert		; vergleiche Beschleunigungszhler
	  	brne 	lichtset			; wenn ungleich weiter mit lichtset
	  	clr 	IncCNT				; lsche Beschleunigungszhler
	  	inc 	V_IST				; erhhe Speed
	  	rjmp 	PWMset		 		; weiter

DEC_V: 	inc		IncCNT
		cp 		IncCNT,DECWert		; vergleiche Beschleunigungszhler
	  	brne 	lichtset			; wenn ungleich weiter mit lichtset
	  	clr 	IncCNT				; lsche Beschleunigungszhler	
		dec 	V_IST				; verringere Speed
	  	  
PWMset: tst  	V_IST				; ist Speed 0
	  	brne 	PWMout				; wenn nicht weiter mit...
									; ansonsten Timer/Counter1 PWM vom 
        out  	TCCR1,PWM_aus		; Ausgang trennen 
		rjmp	lichtset
		  
PWMout:	out  	OCR1A,V_IST			; Wert fr Speed nach Timer 1 Compare 
	  	out  	TCCR1,PWM_an		; Timer/Counter1 PWM Ausgang ein
Richtung_setzen:
		sbrs 	DCCReg,Richtung		; wenn Fahrtrichtung vorwrts 1x springen
	  	rjmp 	REVout				; sonst gehe zu rckwrts
	  	sbi  	PORTB,PB0			; setze Ausgang auf vorwrts
		rjmp	lichtset
REVout:	cbi  	PORTB,PB0			; setze Ausgang auf rckwrts

Lichtset:							; Licht und FktAusgnge setzen
		sbrc	DCCReg,FL			; wenn Licht aus 1x springen
		rjmp	Lichtan				; wenn an gehe zu...
		cbi		portb,3				; lsche lichtausgnge
		cbi		portb,4
		rjmp	FAset				; gleich weiter mit...
Lichtan:
		inc		PWMzaehler			; Zlher erhhen
		cpi		PWMzaehler,100		; wenn 100 dann wieder auf 0 setzen
		brne	PC+2
		clr		PWMzaehler
		cbi		portb,3  			; Licht Rckwrts aus
		cbi		portb,4				; Licht Vorwrts aus
		cp		PWMzaehler,FL_PWM	; Vergleich mit PWM Wert
		brlo	FAset				; wenn Zaehler kleiner berspringen					
		sbrs	DCCReg,richtung		; 1x springen bei vorwrts
		rjmp	Lichtrueckwaerts
Lichtvorwaerts:
		sbi		portb,3				; Licht Vorwrts an
		rjmp	FAset				; weiter mit...
Lichtrueckwaerts:
		sbi		portb,4				; Licht Rckwrts an

FAset:								; Funktionsausgnge setzen
;		sbrc	DCCReg,F1			; wenn F1 gesetz:
;		sbi		portb,4				; PB4 einschalten
;		sbrs	DCCReg,F1			; wenn F1 aus
;		cbi		portb,4				; PB4 ausschalten
;		sbrc	DCCReg,F2			; wenn F2 gesetz:
;		sbi		portb,5				; PB5 einschalten
;		sbrs	DCCReg,F2			; wenn F2 aus
;		cbi		portb,5				; PB5 ausschalten
;		sbrc	DCCReg,F3			; wenn F3 gesetz:
;		sbi		portb,6				; PB6 einschalten
;		sbrs	DCCReg,F3			; wenn F3 aus
;		cbi		portb,6				; PB6 ausschalten
;		sbrc	DCCReg,F4			; wenn F4 gesetz:
;		sbi		portb,7				; PB7 einschalten
;		sbrs	DCCReg,F4			; wenn F4 aus
;		cbi		portb,7				; PB7 ausschalten
		out		sreg,fktsreg		; sreg wiederherstellen
		ret							; Rcksprung

;************************************************************************************************
;*					Hauptprogramm																*
;************************************************************************************************

Reset:								; Programm Kaltstart
	  	cli            				; alle Interrupts ausschalten

		ldi  	temp,0b00011011		; PB0/1/3/4 als Ausgang, PB2/5 als Eingang
	  	out  	DDRB,temp
		clr		null				; alles auf low
		out		portb,null
 
	  	ldi  	TEMP,0b01000011		; Bit0/1 Interupt Sense Control
	  	out  	MCUCR,TEMP			; 0 0 Low Level of INT0 generate Interrupt
									; 0 1 jeder Wechsel of INT0
									; 1 0 die fallende Flanke of INT0
									; 1 1 die steigende Flanke von INT0 erzeugt Interrupt
									; Bit3/4 verschiedene Sleep Modes
									; Bit5 Sleep Enable
									; Bit6 global Pull-up PortB aus
	  
	  	ldi  	TEMP,0b01000000		; Bit5 PIN change Interrupt enable
	  	out  	GIMSK,TEMP			; Bit6 INT0 Interrupt enable

	  	ldi		TEMP,0b00000010		; Bit1 Timer/Counter0 Overflow Interrupt Enable
	  	out		TIMSK,TEMP			; Bit2 Timer/Counter1 Overflow Interrupt Enable 
									; Bit6 Timer/Counter1 Output Compare Interrupt Enable	  
	  	ldi  	TEMP,0b11111000		; Timer/Counter1 PWM
        out  	OCR1B,TEMP			; OCR1B mit 248 (5x1und unten3x0 eingeschoben=11111000=V_max_wert)
;	  	ldi  	TEMP,0b01000101		; 0 0 0 0 Timer/Counter1 is stopped.
;       out  	TCCR1,TEMP			; 0 0 0 1 CK*16 (=PCK)
                     				; 0 0 1 0 CK*8 (=PCK/2)
									; 0 0 1 1 CK*4 (=PCK/4)
									; 0 1 0 0 CK*2 (=PCK/8)
									; 0 1 0 1 CK fr PWM mit 6452 Hz
									; 0 1 1 0 CK/2 fr PWM mit 3226 Hz
									; 0 1 1 1 CK/4 fr PWM mit 1613 Hz
									; 1 0 0 0 CK/8 fr PWM mit 806 Hz
									; 1 0 0 1 CK/16 fr PWM mit 403 Hz
									; 1 0 1 0 CK/32 fr PWM mit 201 Hz
									; 1 0 1 1 CK/64 fr PWM mit 100 Hz
									; 1 1 0 0 CK/128 fr PWM mit 50 Hz
									; 1 1 0 1 CK/256
									; 1 1 1 0 CK/512
									; 1 1 1 1 CK/1024

		ldi		eead,EEPWM_Peri		; PWM Teiler fr Motor laden
		rcall	eepromlesen
		mov		PWM_aus,temp		; in PWM_aus Variable kopieren
		ldi		temp,5				; PWM Teiler berechnen und ablegen
		add		PWM_aus,temp		; (siehe oben teiler 0101 = CK = 6452Hz usw)
		sbr		PWM_aus,0b01000000	; 
		mov		PWM_an,PWM_aus		; PWM_an Variable setzen
		sbr		PWM_an,0b00100000
		out		TCCR1,PWM_aus		; setze PWM Teiler und trenne Ausgang ab
		out 	TCNT1,null			; lsche Timer 1
	  	            
	  	ldi  	TEMP,0b01000000		; lsche externen Interupt 0
	  	out 	GIFR,TEMP
	  
	  	ldi  	TEMP,0b00000010		; lsche Timer Interupt
	  	out  	TIFR,TEMP 	  

	  	ldi 	eead,EEOSCCAL		; hole Oszillator Calibrierung
	  	rcall 	eepromlesen
	  	out 	OSCCAL,temp
	  							  
	  	ldi 	eead,EEAdresse		; hole Adresse
	  	rcall 	eepromlesen
	  	mov 	Adresse,Temp

	  	ldi 	eead,EEStart		; hole Startspannung
	  	rcall 	eepromlesen
	  	mov 	V_start,Temp
	  
	  	ldi 	eead,EEIncWert		; hole Beschleunigung
	  	rcall 	eepromlesen
	  	mov 	IncWert,Temp
		inc		IncWert				; 0=aus auf 1 setzten( kleinster wert ist 1 im Programmablauf)
	  
	  	ldi 	eead,EEDecWert		; hole Verzgerung
	  	rcall 	eepromlesen
	  	mov 	DecWert,Temp
		inc		DecWert				; wie incwert
	  
	  	ldi 	eead,EEV_max		; hole Hchstgeschwindigkeit
	  	rcall 	eepromlesen
	  	mov 	V_max,Temp

		ldi 	eead,EEFL_PWM		; hole Licht PWM Wert
	  	rcall 	eepromlesen
	  	mov 	FL_PWM,Temp
	  
	  	ldi 	eead,EEcv29			; Konfig bits holen 
	  	rcall 	eepromlesen
	  	bst 	Temp,1				; 14/28 Fahrstufen
	  	bld 	DCCReg,Fahrstufen
	  	bst 	Temp,0				; Reversierbetrieb
	  	bld 	DCCReg,Richtung_inv
		bst		Temp,6				; CV29 bit6 holen und kurz in T bit merken
		clr		flag				; flag setzen erst lschen
		sbr 	Flag,(1<<resetbit)	; dann setze resetbit
		bld		flag,ProgSw1		; dann fge T bit ein
		bst		Temp,5				; CV29 bit5 holen und
		bld		flag,ProgSw2		; und ablegen
		bst		Temp,4				; CV29 bit4 holen
		bld		flag,Vmerken		; und ablegen

		cbr		DCCReg,(1<<Fl)		; Licht aus damit nicht aufflackert bei stromunterbrechung
	  	clr 	TEMP				; bereinige alle bentigten Register
		clr		V_soll
		clr		V_ist
	  
	  	sei 						; aktiviere alle gesetzten Interupts

		sbrs	flag,Vmerken		; wenn V merken an 1x springen
		rjmp	Hauptschleife		; sonst gehe gleich zu..
		ldi		eead,EEad_V_ist		; EEPROM Zelle fr EEV_ist holen
		rcall	eepromlesen
		mov		EEV_ist,temp
		mov		eead,EEV_ist		; letzte Vsoll holen
		rcall 	eepromlesen
		bst		temp,0				; richtung holen (wurde in bit0 berschrieben)
		bld		DCCreg,Richtung		; und ablegen
		cbr		temp,0b00000001		; danach aus v_ist wieder entfernen
		mov		V_ist,temp			; und V_ist setzen
PWMstartset: 
		tst  	V_IST				; ist Speed 0
	  	brne 	PWMstartout			; dann 0 ansonsten Out
									; Timer/Counter1 PWM vom 
		rjmp	Hauptschleife		; und weiter zur..
PWMstartout:
	  	out  	OCR1A,V_IST			; Wert fr Speed nach Timer 1 Compare
	  	out  	TCCR1,PWM_an		; Timer/Counter1 PWM Ausgang ein
		sbrs 	DCCReg,Richtung		; wenn Fahrtrichtung vorwrts 1x springen
	  	rjmp 	REVstartout			; gehe zu Ausgang auf rckwrts setzen
	  	sbi  	PORTB,PB0			; setze Ausgang auf vorwrts
		rjmp	Hauptschleife		; und weiter zur..
REVstartout:
		cbi  	PORTB,PB0			; setze Ausgang auf rckwrts

Hauptschleife:
	  	clr  	Bitzaehler     		; loesche den BitCounter wieder
        clr  	byteA				; loesche die Empfangsregister wieder
        clr 	byteB
        clr  	byteC
        clr  	byteD
        clr  	byteE
        clr  	byteF
		cbr  	Flag,(1<<neuesbit)+(1<<bit)+(1<<keinDCC)
  	
PREAMBLE: 							; Prfe Prambel nach NMRA-DCC
		rcall	FKTset				; hier in Wartezeit Funktionen setzen
	  	sbrs 	Flag,neuesbit		; wenn kein neues Bit dann
	  	rjmp 	PC-2				; Schleife
	  	cbr 	Flag,(1<<neuesbit)	; lsche flag fr neues Bit
	  	sbrs 	Flag,bit			; wenn das neue Bit = 1 dann erhhe Zhler
	  	rjmp 	PreEND				; ansonsten Signal fr Prambel Ende
	  	inc 	Bitzaehler			;
	  	rjmp 	PREAMBLE			; Schleife

PreEND: cpi 	Bitzaehler,10		; wenn Bitzhler kleiner 10
	  	brlo 	PreFehl				; dann Fehler		  
	  	clr 	Bitzaehler			; lsche Zhler
	  	rjmp 	lesebyteA  			; hole Adresse (kurze NMRA-Adresse )
 
PreFehl:rjmp 	hauptschleife		; Endlosschleife

lesebyteA: 							; hole erstes von bis zu sechs Bytes
		rcall	FKTset				; hier in Wartezeit Funktionen setzen
	  	sbrs 	Flag,neuesbit		; wenn kein neues Bit dann
	  	rjmp 	PC-2				; Schleife
	  	cbr 	Flag,(1<<neuesbit)	; lsche flag fr neues Bit
	  	cpi 	Bitzaehler,8		; vergleiche mit Bytelnge		;
	  	breq 	byteAENDE			; und springe wenn Byte eingelesen
	  	sec							; setzte Carry
	  	sbrs 	Flag,bit			; wenn empfangenes Bit=1 1x springen
	  	clc							; lsche Carry
	  	rol 	byteA				; schiebe Carry von rechts in das Register
	  	inc 	Bitzaehler			; erhhe Zhler
	  	rjmp 	lesebyteA		  	; Schleife
byteAENDE:
		mov 	Temp,byteA			; sichere Byte
	  	sbrc 	Flag,bit			; springe 1x wenn empfangenes Bit=0 
	  	rjmp 	hauptschleife		; sonst Byte nicht korrekt abgeschlossen
	  	clr 	Bitzaehler			; lsche Zhler

lesebyteB:							; hole zweites von bis zu sechs Bytes
		rcall	FKTset				; hier in Wartezeit Funktionen setzen
	  	sbrs 	Flag,neuesbit		; wenn kein neues Bit dann
	  	rjmp 	PC-2				; Schleife
	  	cbr 	Flag,(1<<neuesbit)	; lsche flag fr neues Bit
	  	cpi 	Bitzaehler,8		; vergleiche mit Bytelnge		;
	  	breq 	byteBENDE			; und springe wenn Byte eingelesen
	  	sec							; setzte Carry
	  	sbrs 	Flag,bit			; wenn empfangenes Bit=1 1x springen
	  	clc							; lsche Carry
	  	rol 	byteB				; schiebe Carry von rechts in das Register
	  	inc 	Bitzaehler			; erhhe Zhler
	  	rjmp 	lesebyteB		  	; Schleife
byteBENDE:
		eor 	TEMP,byteB			; bilde XOR	
	  	sbrc 	Flag,bit			; springe 1x wenn empfangenes Bit=0 
	  	rjmp 	hauptschleife		; sonst Byte nicht korrekt abgeschlossen
	  	clr 	Bitzaehler			; lsche Zhler

lesebyteC:							; hole drittes von bis zu sechs Bytes
		rcall	FKTset				; hier in Wartezeit Funktionen setzen
	  	sbrs 	Flag,neuesbit		; wenn kein neues Bit dann
	  	rjmp 	PC-2				; Schleife
	  	cbr 	Flag,(1<<neuesbit)	; lsche flag fr neues Bit
	  	cpi 	Bitzaehler,8		; vergleiche mit Bytelnge		;
	  	breq 	byteCENDE			; und springe wenn Byte eingelesen
	  	sec							; setzte Carry
	  	sbrs 	Flag,bit			; wenn empfangenes Bit=1 1x springen
	  	clc							; lsche Carry
	  	rol 	byteC				; schiebe Carry von rechts in das Register
	  	inc 	Bitzaehler			; erhhe Zhler
	  	rjmp 	lesebyteC		  	; Schleife	 
byteCENDE:
		eor 	TEMP,byteC			; bilde XOR
	  	sbrc 	Flag,bit			; springe 1x wenn empfangenes Bit=0  
	  	rjmp 	Datenpruefen		; ansonsten Telegrammende
	  	clr 	Bitzaehler			; lsche Zhler

lesebyteD: 							; hole viertes von bis zu sechs Bytes
		rcall	FKTset				; hier in Wartezeit Funktionen setzen
	  	sbrs 	Flag,neuesbit		; wenn kein neues Bit dann
	  	rjmp 	PC-2				; Schleife
	  	cbr 	Flag,(1<<neuesbit)	; lsche flag fr neues Bit
	  	cpi 	Bitzaehler,8		; vergleiche mit Bytelnge		;
	  	breq 	byteDENDE			; und springe wenn Byte eingelesen
	  	sec							; setzte Carry
	  	sbrs 	Flag,bit			; wenn empfangenes Bit=1 1x springen
	  	clc							; lsche Carry
	  	rol 	byteD				; schiebe Carry von rechts in das Register
	  	inc 	Bitzaehler			; erhhe Zhler
	  	rjmp 	lesebyteD		  	; Schleife	 
byteDENDE:
		eor 	TEMP,byteD			; bilde XOR
	  	sbrc 	Flag,bit			; springe 1x wenn empfangenes Bit=0  
	  	rjmp 	Datenpruefen		; ansonsten Telegrammende
	  	clr 	Bitzaehler			; lsche Zhler

lesebyteE: 							; hole fnfte von bis zu sechs Bytes
		rcall	FKTset				; hier in Wartezeit Funktionen setzen
	  	sbrs 	Flag,neuesbit		; wenn kein neues Bit dann
	  	rjmp 	PC-2				; Schleife
	  	cbr 	Flag,(1<<neuesbit)	; lsche flag fr neues Bit
	  	cpi 	Bitzaehler,8		; vergleiche mit Bytelnge		;
	  	breq 	byteEENDE			; und springe wenn Byte eingelesen
	  	sec							; setzte Carry
	  	sbrs 	Flag,bit			; wenn empfangenes Bit=1 1x springen
	  	clc							; lsche Carry
	  	rol 	byteE				; schiebe Carry von rechts in das Register
	  	inc 	Bitzaehler			; erhhe Zhler
	  	rjmp 	lesebyteE		  	; Schleife	 
byteEENDE:
		eor 	TEMP,byteE			; bilde XOR
	  	sbrc 	Flag,bit			; springe 1x wenn empfangenes Bit=0  
	  	rjmp 	Datenpruefen		; ansonsten Telegrammende
	  	clr 	Bitzaehler			; lsche Zhler

lesebyteF: 							; hole sechste von bis zu sechs Bytes
		rcall	FKTset				; hier in Wartezeit Funktionen setzen
	  	sbrs 	Flag,neuesbit		; wenn kein neues Bit dann
	  	rjmp 	PC-2				; Schleife
	  	cbr 	Flag,(1<<neuesbit)	; lsche flag fr neues Bit
	  	cpi 	Bitzaehler,8		; vergleiche mit Bytelnge		;
	  	breq 	byteFENDE			; und springe wenn Byte eingelesen
	  	sec							; setzte Carry
	  	sbrs 	Flag,bit			; wenn empfangenes Bit=1 1x springen
	  	clc							; lsche Carry
	  	rol 	byteF				; schiebe Carry von rechts in das Register
	  	inc 	Bitzaehler			; erhhe Zhler
	  	rjmp 	lesebyteF		  	; Schleife
	 
byteFENDE:
		eor 	TEMP,byteF			; bilde XOR
	  	sbrc 	Flag,bit			; springe 1x wenn empfangenes Bit=0 
	  	rjmp 	hauptschleife		; sonst Byte nicht korrekt abgeschlossen

Datenpruefen:						; Datenprfung und bergabe
		rcall	FKTset				; hier in Wartezeit Funktionen setzen
		tst 	TEMP				; prfe temp auf 0			 	 
	  	brne 	pruefFehl			; wenn nicht 0 dann Fehler
	  
	  	sbrc 	Flag,0				; wenn 1 war reset da gehe zu
		rjmp	progmode			; Programmiermodus CV-Lesen, Schreiben, Bitmanipulation
									; sonst normal weiter	  
	  	ldi 	TEMP,0 				; prfe Adresse
	  	cp 		byteA,TEMP			; wenn 0 dann 
	  	breq 	analle				; Broadcast an alle
	  
	  	ldi 	TEMP,255			; prfe Adresse
	  	cp 		byteA,TEMP			; wenn 255 dann
	  	breq 	IDLE				; IDLE an alle
	  
	  	cp  	Adresse,byteA		; wenn eigene Adresse
	  	breq 	pruefenENDE			; dann test OK
	  	  
IDLE:								; alle Decoder sind IDLE (machen nichts bei diesem befehl)
	  	  
pruefFehl:							; wenn Prfsummenfehler
	  	rjmp 	hauptschleife		; Endlosschleife

analle: 							; alle Decoder sind angesprochen
  	  
pruefenENDE:						; Adresse und Prfsumme OK
		rcall	FKTset				; jetzt ersteinmal wieder Funktionen setzen  
	 								; Funktionen auswhlen
	  	mov 	TEMP,byteB			; Daten sichern
	  	andi 	TEMP,0b11100000 	; Datenbits ausblenden
		cpi		temp,0b00000000		; Vergleich auf dcc reset
		breq	zu_dcc_reset		; dec reset beatbeiten
;	  	cpi 	TEMP,0b00100000		; Vergleiche auf erweiterte Operation (enthlt befehl fr 128 Fahrstufen)
;	  	breq 	erweit_Befehle		; wenn ja gehe zu
	  	cpi 	TEMP,0b01000000		; Vergleiche mit V Rckwrts
	  	breq 	V_Calc  			; wenn ja gehe zu
	  	cpi 	TEMP,0b01100000		; Vergleiche mit V Vorwrts
	  	breq 	V_Calc
	  	cpi 	TEMP,0b10000000		; Vergleiche mit Funktionsgruppe F1-F4
	  	breq 	zu_Fgruppe1
									; alles andere wird nicht unterstzt -> verwerfen und zurck
	  	rjmp 	hauptschleife		; Endlosschleife

zu_dcc_reset:
		rjmp	reset				; Dec reset machen

zu_Fgruppe1:
		rjmp	Fgruppe1

V_Calc:								; V Berechnung 14 und 28 Fahrstufen
		rcall	FKTset				; jetzt ersteinmal wieder Funktionen setzen
									; zuerst Richtung schreiben
	  	mov 	TEMP,byteB			; Daten sichern
	  	tst 	V_IST				; ist V ungleich 0 ?
	  	brne 	V_set				; wenn ja gehe zu Fahrstufen setzen
	  	sbrc 	DCCReg,richtung_inv	; wenn Fahrrichtung nicht reversiert 1x springen
        com 	byteB				; ansonsten wird Byte gedreht	
        bst 	byteB,5				; hole Richtungsbit
	  	bld 	DCCReg,richtung		; nach DCC Register
V_set:								; Fahrstufen setzen
		sbrc	DCCReg,fahrstufen	; 0=14 Fahrstufen 1x springen
		rjmp	V_28set				; wenn 28 gehe zu
		bst		temp,4				; Lichtbit holen
		bld		DCCReg,FL			; und in FL ablegen
		cbr		temp,0b11110000		; bits fr kennung, richtung und licht lschen

V_28set:							; ab hier fr 14 und 28 Fahrstufen gleich
		rcall	FKTset				; jetzt ersteinmal wieder Funktionen setzen
		bst		temp,4				; bit0 fr 28 Fahrstufen sichern
		cbr		temp,0b11110000		; bits fr kennung und richtung lschen
		cpi		temp,0b00000001		; auf notstop testen
		breq	Notstop				; wenn gleich gehe zu
		rol		temp				; 1x nachlinks verschieben
		bld		temp,0				; und bit0 wieder einfgen
		mov		byteB,temp			; V_max begrenzung auf 50/62,5%/75/88,5%/100%
		add		temp,byteB			; V auf 37,5%
		add		temp,byteB
		cpi		V_max,4
		breq	V50set
		cpi		V_max,3
		breq	V625set
		cpi		V_max,2
		breq	V75set
		cpi		V_max,1
		breq	V885set
		rcall	FKTset				; jetzt ersteinmal wieder Funktionen setzen
V100set:add		temp,byteB
V885set:add		temp,byteB		
V75set:	add		temp,byteB
V625set:add		temp,byteB	
V50set:	add		temp,byteB
Vstartset:
		cpi		temp,0				; bei stop berspringen
		breq	Vsollset
		cp		temp,V_start		; Startgeschw setzen 
		brsh	Vsollset			; wenn neue V kleiner V_start
		mov		temp,V_start
Vsollset:
		cp		V_soll,temp			; neuen V wert empfangen?
		brne	PC+2				; wenn ja 1x springen
		rjmp	Hauptschleife		; wenn kein neuer wert gehe zurck
		rcall	FKTset				; jetzt ersteinmal wieder Funktionen setzen
		mov		V_soll,temp			; neue soll geschw. ablegen
		sbrs	flag,Vmerken		; wenn V merken an 1x springen
		rjmp	Hauptschleife		; sonst gehe gleich zu..
		ldi		PWMzaehler,99		; vor EEprom schreiben Licht bei PWM Betrieb aus (blitzt sonst auf)
		rcall	FKTset				; jetzt ersteinmal wieder Funktionen setzen
		bst		DCCreg,richtung		; Richtungsbit holen
		bld		temp,0				; und bit0 von Vsoll in Temp berschreiben
		mov		eead,EEV_ist		; V wert im EEprom merken
		rcall	eepromschreiben
	  	rjmp 	hauptschleife		; Endlosschleife

Notstop:
	  	clr 	V_SOLL				; Notstop machen
	  	clr 	V_IST				;
									; Timer/Counter1 PWM vom 
        out  	TCCR1,PWM_aus		; Ausgang trennen
        cbi  	PortB,0				; alles auf Null
	  	rjmp 	hauptschleife		; Endlosschleife		

Fgruppe1:							; Befehle fr FG1 einlesen
		rcall	FKTset				; jetzt ersteinmal wieder Funktionen setzen
		bst		byteB,F1			; F1 bis F4 sichern
		bld		dccreg,F1
		bst		byteB,F2
		bld		dccreg,F2
		bst		byteB,F3
		bld		dccreg,F3
		bst		byteB,F4
		bld		dccreg,F4
		sbrs	DCCReg,fahrstufen	; 1=28 Fahrstufen 1x springen
		rjmp 	hauptschleife		; bei 14 fertig Endlosschleife
		bst		byteB,FL			; bei 28 noch Lichtbit sichern
		bld		dccreg,FL
		rjmp	hauptschleife		; und dann zurck				

progmode:
	  	mov 	TEMP,byteA			; Daten sichern
		cpi		temp,0b00000000		; Vergleich auf dcc reset
		breq	noch_dcc_reset		; dec reset bearbeiten

		cpi 	TEMP,0b01111100		; prfe Funktionscode auf CV Write Byte Schreiben
		breq 	progbyte			; 	
		
		cbr		flag,(1<<resetbit)	; wenn etwas anderes kommt lsche Reset und gehe zu
		rjmp	hauptschleife

noch_dcc_reset:
		rjmp	reset				; gehe nocheimal zu

progbyte:
		cp  	PROGTEST,ByteD   	; vergleiche mit vorherigen Daten (XOR-Byte)
	  	brne 	PROGWRACK			; springe wenn ungleich
		mov		temp,byteB			; teste ob cv 29 geschrieben werden soll?
		cpi		temp,29
		brsh	PROGWRACK			; CVreg grer 29 wird nicht unterstzt -> abbrechen
		sbrs	flag,ProgSw1		; wenn 0
		rjmp	progsw2_test		; gehe zu ...
		ldi		eead,EEcv29			; progschalter wieder auf normal schalten
		rcall	eepromlesen			; es wird genau 1x schreiben in FL PWM erlaubt
		cbr		Temp,0b01000000		; 
		rcall	eepromschreiben		;
		ldi		temp,(EEFL_PWM-1)	; sonst setzte auf FL PWM Byte
		mov		ByteB,temp
		rjmp	prognormal
progSw2_test:
		sbrs	flag,ProgSw2		; wenn 0
		rjmp	prognormal			; gehe zu ...
		ldi		eead,EEcv29			; progschalter wieder auf normal schalten
		rcall	eepromlesen			; es wird genau 1x schreiben in Motor PWM erlaubt
		cbr		Temp,0b00100000		; 
		rcall	eepromschreiben		;
		ldi		temp,(EEPWM_Peri-1)	; sonst setzte auf FL PWM Byte
		mov		ByteB,temp
prognormal:
	  	mov 	eead,ByteB			; setzte Schreib-Adresse
		inc		eead				; weil im eeprom zelle0 unbenutzt bleibt (alles 1x nach obenverschoben)
	  	mov 	TEMP,ByteC			; hole Daten
	  	rcall 	eepromschreiben		; und schreibe ins EEPROM

	  	sbi  	PortB,PB1			; setze einen Motorausgang
	  
PROGBeschtaetigung:	  				; Acknowledge
	  	cli            				; alle Interrupts ausschalten
	  	ldi  	TEMP,63   			; lade den Timer/Counter1
		out  	TCNT1,TEMP			; setze Zeit fr Acknowlege
	  	ldi  	TEMP,0b00001100		; 1 1 0 0 CK/128 fr Resetzeit und ACK
        out  	TCCR1,TEMP			; 
        in   	TEMP,TIFR			; hole Timer Interuptflag
	  	sbrs 	TEMP,TOV1			; wenn Ackzeit abgelaufen
	  	rjmp 	PC-2				; Schleife
        ldi  	TEMP,0b00000110		; lsche Timer Interupts
	  	out  	TIFR,TEMP
        out 	PortB,null  		; lsche Motorausgang
		cbr		flag,(1<<resetbit)	; lsche Reset und dann normal weiter
        sei            				; alle Interrupts wieder einschalten

PROGWRACK: 
		mov 	PROGTEST,ByteD		; Datencheck		  				
        rjmp 	Hauptschleife		; Endlosschleife
		
	
.ESEG

;CV Register im eeprom vom tiny15 Lokdec

.db $ff
.db 0b00000001	;CVregister 1 = Dekoderaddresse
.db $1			;CVregister 2 = Startspannung 0-255 0 funktion aus 255 immer V_max
.db $0			;CVregister 3 = Beschleunigungsrate ca 0-255 zeitstufen
.db $0			;CVregister 4 = Verzgerungsrate ca 0-255 zeitstufen
.db $4			;CVregister 5 = V_max Begrenzung auf 4=50% 3=62,5% 2=75% 1=88,5% 0=100%
.db 70			;CVregister 6 = Licht PWM dunkel Zeit in %	(0 immer an 99 = 99% aus)	
.db 32			;CVregister 7 = Zeiger auf Vist Speicherplatz Standart ist 32
.db $ff
.db 5			;CVregister 9 = Motor PWM Periode (0=6452Hz 1=3226Hz bis 7=50Hz) 5=202Hz
.db $ff
.db $ff
.db $ff
.db $ff
.db $ff
.db $ff
.db $ff
.db $ff	
.db $ff
.db $ff
.db $ff
.db $ff
.db $ff
.db $ff
.db $ff
.db $ff
.db $ff
.db $ff
.db $ff
.db 0b00010010	;CVregister 29 Konfigurationsregister (zum Licht PWM programmieren+64=82 Motor PWM+32=50)
				;Funktionen:	bit0=1 Richtungstausch
				;(18)			bit1=0 14/ =1 28 Fahrstufen
				;				bit4=1 Vsoll in EEprom merken
				;				bit5=1 nchster ProgBefehl Motor PWM
				;				bit6=1 nchster ProgBefehl Licht PWM
.db $ff
.db $ff
.db $0			;bit1-7 Vist bit0 Richtung wenn in CV 29 gefordert
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF, $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF, $FF,$FF,$FF,$FF,$FF,$FF,$FF,$70			;Osccal byte

